/*
 * Copyright (c) 2009, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef GRIPPER_CLICK_DISPLAY_H
#define GRIPPER_CLICK_DISPLAY_H

#include "rviz/display.h"

#include <actionlib/server/simple_action_server.h>

#include "pr2_gripper_click/GripperClickPickupAction.h"
#include "pr2_gripper_click/gripper_click_3d_gripper.h"
#include "pr2_gripper_click/gripper_click_grasp_adjust.h"
#include "pr2_gripper_click/gripper_click_line.h"
#include "pr2_gripper_click/gripper_click_dot.h"
#include "pr2_gripper_click/gripper_click_place_cartesian.h"
#include "pr2_gripper_click/gripper_click_pickup_cartesian.h"


namespace pr2_gripper_click
{

// will start an action server, instatiate a gripper click ui
// which can hold plugins for the specific action
template<class ActionClass>
class GripperClickDisplay : public rviz::Display
{
public:

  typedef typename ActionClass::_action_result_type::_result_type ResultType;
  typedef typename ActionClass::_action_goal_type::_goal_type GoalType;

  GripperClickDisplay( std::string action_topic, const std::string& name, rviz::VisualizationManager* manager );
  
  virtual ~GripperClickDisplay();

  virtual void targetFrameChanged() {}
  virtual void fixedFrameChanged() {}

  virtual void update(float wall_dt, float ros_dt);

  virtual void onEnable();

  virtual void onDisable();

  //start listening to action goals
  void startActionServer();

  //stop action server, cancel current goal & hide if necessary
  void stopActionServer();

  // callback for new action server goals
  virtual void acceptNewGoal();

  virtual void preemptAction();
  virtual void abortAction();

protected:

  GripperClickRvizUI<ActionClass> *gripper_click_rviz_ui_;

  actionlib::SimpleActionServer<ActionClass> *action_server_;

  std::string action_topic_;
};

//specialized displays for pickup / place
//note: GripperClickDisplay<GripperClickPickupAction> etc. are instantiated in the cpp

class GripperClickPickupDisplay : public GripperClickDisplay<GripperClickPickupAction>
{
public:
  GripperClickPickupDisplay( const std::string& name, rviz::VisualizationManager* manager ) :
    GripperClickDisplay<GripperClickPickupAction>( "gripper_click_pickup_ui", name, manager ) {};

  virtual void onEnable()
  {
    GripperClickDisplay<GripperClickPickupAction>::onEnable();
    gripper_click_rviz_ui_->addPlugin( new GripperClickPickupCartesian() );
    gripper_click_rviz_ui_->addPlugin( new GripperClick3DGripper() );
    gripper_click_rviz_ui_->addPlugin( new GripperClickGraspAdjust() );
    gripper_click_rviz_ui_->addPlugin( new GripperClickLine() );
    gripper_click_rviz_ui_->SetTitle( wxString::FromAscii("Pick up object") );
  }
};

class GripperClickPlaceDisplay : public GripperClickDisplay<GripperClickPlaceAction>
{
public:
  GripperClickPlaceDisplay( const std::string& name, rviz::VisualizationManager* manager ) :
    GripperClickDisplay<GripperClickPlaceAction>( "gripper_click_place_ui", name, manager ) {};

  virtual void onEnable()
  {
    GripperClickDisplay<GripperClickPlaceAction>::onEnable();
    gripper_click_rviz_ui_->addPlugin( new GripperClickPlaceCartesian() );
    gripper_click_rviz_ui_->addPlugin( new GripperClickDot() );
    gripper_click_rviz_ui_->SetTitle( wxString::FromAscii("Place object") );
  }
};

}

#endif
