/*
 * Copyright (c) 2009, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef GRIPPER_CLICK_GRASP_ADJUST
#define GRIPPER_CLICK_GRASP_ADJUST

#include "pr2_gripper_click/gripper_click_rviz_ui.h"
#include "rviz_interaction_tools/gripper.h"

#include <tf/transform_datatypes.h>

#include <stereo_msgs/DisparityImage.h>
#include <sensor_msgs/CameraInfo.h>

#include <actionlib/client/simple_action_client.h>
#include <pr2_grasp_adjust/GraspAdjustAction.h>

typedef actionlib::SimpleActionClient<pr2_grasp_adjust::GraspAdjustAction> ActionClient;



namespace Ogre {
class SceneNode;
}

namespace rviz_interaction_tools {
class DisparityRenderer;
}

namespace pr2_gripper_click {

class GripperClickGraspAdjust : public GripperClickPlugin<GripperClickPickupAction>
{

public:

  GripperClickGraspAdjust();

  ~GripperClickGraspAdjust();

  // do the actual initialization, instantiate ogre elements etc.
  virtual void init( rviz::VisualizationManager *vis_manager, Ogre::SceneManager* scene_manager, Ogre::SceneNode* camera_node );

  virtual bool setGoal( const GripperClickPickupGoal::ConstPtr &goal );

  virtual bool getResult(GripperClickPickupResult &result);

  virtual void update(float wall_dt, float ros_dt);

  // @return true if a result would already be available
  virtual bool hasValidResult() { return valid_transform_; }

  // make the orge content visible / invisible
  virtual void show( );
  virtual void hide( );

  virtual void getGripperParams( bool &visible, Ogre::Vector3 &position, Ogre::Quaternion &orientation, float &angle );

  virtual void onRenderWindowMouseEvents( wxMouseEvent& event, Ogre::Ray &mouse_ray );

protected:

  struct ClickInfo
  {
    ClickInfo() : down_(false), x_(0), y_(0), ref_(0.0) {}
    bool down_;
    int x_, y_;
    float ref_;
  };


  void requestGraspAdjustAction();
  void receiveGraspAdjustAction(const actionlib::SimpleClientGoalState &state,
                                const pr2_grasp_adjust::GraspAdjustResultConstPtr &result);

  int gripper_index_;
  std::vector<float> gripper_angle_;
  std::vector<float> gripper_score_;
  std::vector<float> gripper_roll_;
  std::vector<float> gripper_approach_;
  std::vector<btTransform> gripper_transform_;

  ClickInfo rb_, mb_, lb_, dlb_;

  bool valid_transform_;

  //rviz_interaction_tools::Gripper* gripper_;
  Ogre::ManualObject* line_object_;

  stereo_msgs::DisparityImage disparity_image_;
  sensor_msgs::CameraInfo camera_info_;

  Ogre::SceneNode *scene_node_;

  ActionClient action_client_;
};

}

#endif
