/*
 * Copyright (c) 2009, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef GRIPPER_CLICK_CARTESIAN
#define GRIPPER_CLICK_CARTESIAN

#include "pr2_gripper_click/gripper_click_rviz_ui.h"
#include "rviz_interaction_tools/cartesian_control.h"
#include "ogre_tools/point_cloud.h"

namespace Ogre {
class ManualObject;
}

namespace pr2_gripper_click {

class GripperClickPlaceCartesian : public GripperClickPlugin<GripperClickPlaceAction>
{

public:

  GripperClickPlaceCartesian();

  // do the actual initialization, instantiate ogre elements etc.
  virtual void init( rviz::VisualizationManager *vis_manager, 
                     Ogre::SceneManager* scene_manager, 
                     Ogre::SceneNode* camera_node );

  virtual bool setGoal( const GripperClickPlaceGoal::ConstPtr &goal );

  virtual bool getResult( GripperClickPlaceResult &result );

  virtual void update(float wall_dt, float ros_dt);

  // @return true if a result would already be available
  virtual bool hasValidResult();

  // make the orge content visible / invisible
  virtual void show( );
  virtual void hide( );

  virtual void onRenderWindowMouseEvents( wxMouseEvent& event, Ogre::Ray &mouse_ray );

  virtual void getGripperParams( bool &visible, Ogre::Vector3 &position, Ogre::Quaternion &orientation, float &angle );

protected:

  stereo_msgs::DisparityImage disparity_image_;
  sensor_msgs::CameraInfo camera_info_;

  rviz_interaction_tools::CartesianControl *cartesian_control_;

  Ogre::SceneNode *gripper_scene_node_;
  Ogre::SceneNode *main_win_object_scene_node_;

  Ogre::SceneNode *camera_node_;

  bool initialized_;

  ogre_tools::PointCloud *point_cloud_;
  ogre_tools::PointCloud *main_win_point_cloud_;

  // Orientation of the object coordinate frame, relative to the fixed frame
  Ogre::Quaternion object_orientation_;

  // pose of the gripper relative to the object frame
  Ogre::Vector3 gripper_position_;
  Ogre::Quaternion gripper_orientation_;
};

}

#endif
